import { TemplatePortal, ComponentPortal, BasePortalOutlet, CdkPortalOutlet, PortalModule } from '@angular/cdk/portal';
import { Injectable, EventEmitter, TemplateRef, Component, ComponentFactoryResolver, ApplicationRef, Injector, Renderer2, Output, Input, HostListener, ViewContainerRef, ElementRef, ViewChild, HostBinding, ContentChildren, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { debounceTime, first } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

class WalkthroughText {
    constructor() {
        this.previous = 'Previous';
        this.next = 'Next';
        this.close = 'Close';
    }
}

const booleanValue = (value) => {
    return value === '' || value === 'true' || value === true;
};
class WalkthroughEvent {
    constructor(component, focusElement) {
        this.component = component;
        this.focusElement = focusElement;
    }
}
class WalkthroughMargin {
    constructor(top = 0, right, bottom, left) {
        this.top = top;
        this.right = right;
        this.bottom = bottom;
        this.left = left;
        if (right === undefined) {
            this.right = top;
        }
        if (bottom === undefined) {
            this.bottom = top;
        }
        if (left === undefined && right === undefined) {
            this.left = top;
        }
        else if (left === undefined) {
            this.left = right;
        }
    }
    static parsePoints(points) {
        let pointsPx;
        if (points.match(/^\d+(?:\s+\d+)*$/)) {
            const split = points.split(/\s+/).map(i => parseFloat(i));
            pointsPx = new WalkthroughMargin(split[0], split[1], split[2], split[3]);
        }
        return pointsPx || new WalkthroughMargin();
    }
}

class WalkthroughService {
    constructor() {
        this._overflowRegex = /(auto|scroll)/;
    }
    retrieveCoordinates(element, margin) {
        const clientrect = element.getBoundingClientRect();
        const style = window.getComputedStyle(element);
        const coordinates = {
            top: clientrect.top - (margin ? margin.top : 0),
            height: clientrect.height,
            width: clientrect.width,
            left: clientrect.left - (margin ? margin.left : 0),
            margin: {
                top: parseFloat(style.marginTop),
                right: parseFloat(style.marginRight),
                bottom: parseFloat(style.marginBottom),
                left: parseFloat(style.marginLeft),
            },
        };
        coordinates.top += this.getTop();
        return coordinates;
    }
    getTop() {
        return window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop;
    }
    getDocumentHeight() {
        // Height of entire body : https://stackoverflow.com/a/1147768
        const body_height = Math.max(document.body.scrollHeight, document.body.offsetHeight, document.documentElement.clientHeight, document.documentElement.scrollHeight, document.documentElement.offsetHeight);
        return Math.max(this.getHeightOfPage() + this.getTop(), body_height);
    }
    scrollIntoViewIfOutOfView(element) {
        const topOfPage = this.getTop();
        const heightOfPage = this.getHeightOfPage();
        let elementY = 0;
        let elementH = 0;
        let parent = element;
        while (parent && parent !== document.body) {
            elementY += parent.offsetTop;
            parent = parent.offsetParent;
        }
        elementH = element.offsetHeight;
        if (topOfPage + heightOfPage < elementY + elementH) {
            element.scrollIntoView(false);
        }
        else if (elementY < topOfPage) {
            element.scrollIntoView(true);
            window.scrollBy(0, -30);
        }
        else {
            // test of overflow element
            let current = element;
            while (current && current !== document.body) {
                parent = this.getScrollParent(current);
                if (current.offsetTop + current.offsetHeight - parent.scrollTop > parent.offsetHeight ||
                    current.offsetLeft + current.offsetWidth - parent.scrollLeft > parent.offsetWidth) {
                    element.scrollIntoView();
                    window.scrollBy(0, -30);
                    break;
                }
                current = parent;
            }
        }
    }
    getScrollParent(element) {
        let scrollParent;
        let style = getComputedStyle(element);
        const excludeStaticParent = style.position === 'absolute';
        if (style.position !== 'fixed') {
            let parent = element.parentElement;
            while (parent && parent !== document.body) {
                style = getComputedStyle(parent);
                if (!(excludeStaticParent && style.position === 'static') &&
                    this._overflowRegex.test(style.overflow + style.overflowY + style.overflowX)) {
                    scrollParent = parent;
                    break;
                }
                parent = parent.parentElement;
            }
        }
        return scrollParent || document.body;
    }
    getHeightOfPage() {
        return window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight;
    }
}
WalkthroughService.decorators = [
    { type: Injectable }
];

let nextUniqueId$1 = 0;
const noInstanceWarn = 'No instance of walkthroughContainer.';
const anoterWktOnGoing = 'Another walkthrough is ongoing. Please close it before opening a new one.';
class WalkthroughComponent {
    constructor(_componentFactoryResolver, _applicationRef, _injector, _renderer, _walkthroughService) {
        this._componentFactoryResolver = _componentFactoryResolver;
        this._applicationRef = _applicationRef;
        this._injector = _injector;
        this._renderer = _renderer;
        this._walkthroughService = _walkthroughService;
        this.closed = new EventEmitter();
        this.finished = new EventEmitter();
        this.ready = new EventEmitter();
        this.typeSelector = 'element';
        this.contentStyle = 'draken';
        this.animation = 'none';
        this.animationDelays = 0;
        this.scrollOnTarget = true;
        this._uid = `walkthrough-${nextUniqueId$1++}`;
        this._readyHasBeenEmited = false;
        this._display = false;
        this._hasHighlightAnimation = false;
        this._hasBackdrop = false;
        this._hasGlow = false;
        this._hasFinish = false;
        this._hasArrow = false;
        this._hasCloseButton = false;
        this._hasCloseAnywhere = true;
        this._disabled = false;
        this._marginZonePx = new WalkthroughMargin();
        this._alignContent = 'left';
        this._verticalAlignContent = 'top';
        this._contentSpacing = 0;
        this._verticalContentSpacing = 50;
        this._onContainerInit = new Subject();
        this._onResize = new Subject();
        this._notScrollOnResize = true;
        this._resizeDelays = 200;
        this._onResize.pipe(debounceTime(this._resizeDelays)).subscribe(() => {
            var _a, _b;
            const instance = this._getInstance();
            if ((instance === null || instance === void 0 ? void 0 : instance.ongoing) && this._display && !((_b = (_a = WalkthroughComponent._walkthroughContainer) === null || _a === void 0 ? void 0 : _a.instance) === null || _b === void 0 ? void 0 : _b.pause)) {
                this._elementLocations(!this._notScrollOnResize);
                setTimeout(() => {
                    this._elementLocations(!this._notScrollOnResize);
                }, 50);
            }
        });
    }
    get marginZone() {
        return this._marginZone;
    }
    set marginZone(points) {
        if (this._marginZone !== points) {
            if (points === null) {
                this._marginZone = null;
            }
            this._marginZonePx = WalkthroughMargin.parsePoints(points);
            if (this._marginZonePx !== null) {
                this._marginZone = points;
            }
        }
    }
    get arrowColor() {
        return this._arrowColor;
    }
    set arrowColor(color) {
        if (this._arrowColor !== color) {
            this._arrowColor = color;
            if (this._getInstance()) {
                this._getInstance().arrowColor = this._arrowColor;
            }
        }
    }
    get notScrollOnResize() {
        return this._notScrollOnResize;
    }
    set notScrollOnResize(value) {
        this._notScrollOnResize = booleanValue(value);
    }
    get id() {
        return this._id;
    }
    set id(value) {
        this._id = value || this._uid;
    }
    get alignContent() {
        return this._alignContent;
    }
    set alignContent(value) {
        if (this._alignContent !== value) {
            this._alignContent = value;
            this._updateElementPositions(this._getInstance());
        }
        else {
            this._alignContent = value;
        }
    }
    get verticalAlignContent() {
        return this._verticalAlignContent;
    }
    set verticalAlignContent(value) {
        if (this._verticalAlignContent !== value) {
            this._verticalAlignContent = value;
            this._updateElementPositions(this._getInstance());
        }
        else {
            this._verticalAlignContent = value;
        }
    }
    get contentSpacing() {
        return this._contentSpacing;
    }
    set contentSpacing(value) {
        value = Math.max(WalkthroughComponent.minimalMargin, value);
        if (this._contentSpacing !== value) {
            this._contentSpacing = value * 1;
            this._updateElementPositions(this._getInstance());
        }
        else {
            this._contentSpacing = value * 1;
        }
    }
    get verticalContentSpacing() {
        return this._verticalContentSpacing;
    }
    set verticalContentSpacing(value) {
        value = Math.max(WalkthroughComponent.minimalMargin, value);
        if (this._verticalContentSpacing !== value) {
            this._verticalContentSpacing = value * 1;
            this._updateElementPositions(this._getInstance());
        }
        else {
            this._verticalContentSpacing = value * 1;
        }
    }
    get closeButton() {
        return this._hasCloseButton;
    }
    set closeButton(value) {
        this._hasCloseButton = booleanValue(value);
    }
    get closeAnywhere() {
        return this._hasCloseAnywhere;
    }
    set closeAnywhere(value) {
        this._hasCloseAnywhere = booleanValue(value);
    }
    get showArrow() {
        return this._hasArrow;
    }
    set showArrow(value) {
        this._hasArrow = booleanValue(value);
    }
    get finishButton() {
        return this._hasFinish;
    }
    set finishButton(value) {
        this._hasFinish = booleanValue(value);
    }
    get focusHighlightAnimation() {
        return this._hasHighlightAnimation;
    }
    set focusHighlightAnimation(value) {
        this._hasHighlightAnimation = booleanValue(value);
    }
    get focusBackdrop() {
        return this._hasBackdrop;
    }
    set focusBackdrop(value) {
        this._hasBackdrop = booleanValue(value);
    }
    get focusGlow() {
        return this._hasGlow;
    }
    set focusGlow(value) {
        this._hasGlow = booleanValue(value);
    }
    get disabled() {
        return this._disabled;
    }
    set disabled(value) {
        this._disabled = value;
        const instance = this._getInstance();
        if (instance) {
            setTimeout(() => {
                instance.hasPrevious = this._hasPreviousStep(instance);
                instance.hasNext = this._hasNextStep(instance);
                if (!instance.hasNext) {
                    instance.hasFinish = true;
                }
                else {
                    instance.hasFinish = instance.parent.finishButton;
                }
            }, 50);
        }
    }
    static walkthroughStop() {
        if (WalkthroughComponent._walkthroughContainer) {
            WalkthroughComponent._walkthroughContainer.instance.stop();
        }
    }
    static walkthroughHasShow() {
        return WalkthroughComponent._walkthroughContainer
            ? WalkthroughComponent._walkthroughContainer.instance.show
            : false;
    }
    static walkthroughHasPause() {
        return WalkthroughComponent._walkthroughContainer
            ? WalkthroughComponent._walkthroughContainer.instance.pause
            : false;
    }
    static walkthroughContinue() {
        if (WalkthroughComponent._walkthroughContainer && WalkthroughComponent._walkthroughContainer.instance.parent) {
            const visible = WalkthroughComponent._walkthroughContainer.instance.parent._checkVisibility();
            if (visible) {
                WalkthroughComponent._walkthroughContainer.instance.continue();
            }
            else {
                // we can't open the walkthrough anymore, the focusElementSelector is no more visible
                // so we update the flag "ongoing"
                WalkthroughComponent._walkthroughContainer.instance.ongoing = false;
            }
        }
    }
    static walkthroughNext() {
        if (WalkthroughComponent._walkthroughContainer) {
            WalkthroughComponent._walkthroughContainer.instance.next();
        }
    }
    static walkthroughPrevious() {
        if (WalkthroughComponent._walkthroughContainer) {
            WalkthroughComponent._walkthroughContainer.instance.previous();
        }
    }
    resize() {
        this._onResize.next();
    }
    ngAfterViewInit() {
        // init the Walkthrough element container
        if (!WalkthroughComponent._walkthroughContainer && !WalkthroughComponent._walkthroughContainerCreating) {
            WalkthroughComponent._walkthroughContainerCreating = true;
            setTimeout(() => {
                WalkthroughComponent._walkthroughContainer =
                    this._appendComponentToBody(WalkthroughContainerComponent);
                this._onContainerInit.next();
            });
        }
    }
    next(closedEvent, finishedEvent) {
        if (closedEvent) {
            this.closed = closedEvent;
        }
        if (finishedEvent) {
            this.finished = finishedEvent;
        }
        this.open();
    }
    refresh() {
        if (this._getInstance()) {
            if (!this._getInstance().ongoing) {
                WalkthroughComponent.onRefresh.next(this);
                this._elementLocations();
            }
            else {
                console.warn(anoterWktOnGoing);
            }
        }
        else {
            console.warn(noInstanceWarn);
        }
    }
    open() {
        return new Promise(resolve => {
            if (this._checkVisibility()) {
                if (this._getInstance()) {
                    resolve(this._open());
                }
                else {
                    this._onContainerInit.pipe(first()).subscribe(() => {
                        if (this._getInstance()) {
                            resolve(this._open());
                        }
                        else {
                            console.warn(noInstanceWarn);
                            resolve(false);
                        }
                    });
                }
            }
            else {
                resolve(false);
            }
        });
    }
    /**
     * Do not use this method outside of the library
     */
    loadPrevioustStep() {
        setTimeout(() => {
            WalkthroughComponent.onNavigate.next({ previous: this, next: this.previousStep });
            WalkthroughComponent.onNavigatePrevious.next({ previous: this, next: this.previousStep });
            this.previousStep._next(this.closed, this.finished);
        }, 0);
    }
    /**
     * Do not use this method outside of the library
     */
    loadNextStep() {
        setTimeout(() => {
            WalkthroughComponent.onNavigate.next({ previous: this, next: this.nextStep });
            WalkthroughComponent.onNavigateNext.next({ previous: this, next: this.nextStep });
            this.nextStep._next(this.closed, this.finished);
        }, 0);
    }
    /**
     * Do not use this method outside of the library
     */
    _hide(finishLink = false, closeWalkthrough = true) {
        this._display = false;
        // add CSS to focusElement
        if (this.focusElementCSSClass && this._focusElement) {
            this._renderer.removeClass(this._focusElement, this.focusElementCSSClass);
        }
        if (closeWalkthrough) {
            setTimeout(() => {
                this._getInstance().ongoing = false;
                WalkthroughComponent.onClose.next(this);
                // emit closed event
                this.closed.emit(finishLink);
                if (!this.nextStep) {
                    // emit finished event
                    WalkthroughComponent.onFinish.next(this);
                    this.finished.emit(new WalkthroughEvent(this, this._focusElement));
                }
            }, 20);
        }
    }
    _checkVisibility() {
        if (this.focusElementSelector) {
            const elements = document.querySelectorAll(this.focusElementSelector);
            if (elements.length === 0) {
                return false;
            }
            if (this.visibilityCallback) {
                return this.visibilityCallback();
            }
            for (const el of elements) {
                if (el.offsetParent === null) {
                    return false;
                }
            }
        }
        return true;
    }
    _open() {
        if (!this._getInstance().ongoing) {
            this._getInstance().ongoing = true;
            WalkthroughComponent.onOpen.next(this);
            this._elementLocations();
            return true;
        }
        else {
            console.warn(anoterWktOnGoing);
            return false;
        }
    }
    _show() {
        this._display = true;
    }
    _next(closedEvent, finishedEvent) {
        if (closedEvent) {
            this.closed = closedEvent;
        }
        if (finishedEvent) {
            this.finished = finishedEvent;
        }
        const instance = this._getInstance();
        if (instance && instance.ongoing) {
            instance.ongoing = false;
        }
        this.open();
    }
    _getInstance() {
        return WalkthroughComponent._walkthroughContainer ? WalkthroughComponent._walkthroughContainer.instance : null;
    }
    _appendComponentToBody(component) {
        // create a component reference
        const componentRef = this._componentFactoryResolver.resolveComponentFactory(component).create(this._injector);
        // attach component to the appRef so that so that it will be dirty checked.
        this._applicationRef.attachView(componentRef.hostView);
        // get DOM element from component
        const domElem = componentRef.hostView.rootNodes[0];
        document.body.appendChild(domElem);
        return componentRef;
    }
    _attachWalkthroughContent(componentOrTemplateRef, walkthroughContainer) {
        if (componentOrTemplateRef instanceof TemplateRef) {
            walkthroughContainer.attachTemplatePortal(new TemplatePortal(componentOrTemplateRef, null));
        }
        else {
            walkthroughContainer.attachComponentPortal(new ComponentPortal(componentOrTemplateRef));
        }
    }
    _elementLocations(scroll = true) {
        this._getFocusElement();
        const element = this._focusElement;
        if (element) {
            if (scroll) {
                if (this.scrollOnTarget) {
                    this._walkthroughService.scrollIntoViewIfOutOfView(element);
                }
                // if there is a root element defined (in some cases when position fixed is used, we need to scroll on it)
                if (this.rootElement) {
                    document.querySelector(this.rootElement).scrollIntoView(true);
                }
            }
            this._offsetCoordinates = this._walkthroughService.retrieveCoordinates(element);
            if (this.typeSelector === 'zone') {
                const offsetEndCoordinatesEnd = this._walkthroughService.retrieveCoordinates(this._focusElementEnd);
                this._offsetCoordinates.height =
                    offsetEndCoordinatesEnd.top - this._offsetCoordinates.top + offsetEndCoordinatesEnd.height;
                this._offsetCoordinates.width =
                    offsetEndCoordinatesEnd.left - this._offsetCoordinates.left + offsetEndCoordinatesEnd.width;
            }
        }
        else {
            this._offsetCoordinates = null;
        }
        this._setFocus();
    }
    /**
     *
     */
    _getFocusElement() {
        let focusElements;
        try {
            focusElements = this.focusElementSelector
                ? document.querySelectorAll(this.focusElementSelector)
                : null;
        }
        catch (error) {
            console.error(`#${this.id}@focusElementSelector: '${this.focusElementSelector}' is not a valid selector.\n`, error);
        }
        // getting focus element
        if (focusElements && focusElements.length > 0) {
            if (focusElements.length > 1) {
                // Multiple items fit selector, displaying first visible as focus item in 'element' mode
                const l = focusElements.length;
                for (let i = 0; i < l; i++) {
                    // offsetHeight not of 0 means visible
                    if (focusElements[i].offsetHeight) {
                        this._focusElement = focusElements[i];
                        i = focusElements.length;
                        break;
                    }
                }
                // if typeSelector is by zone, get also the last element
                if (this.typeSelector === 'zone') {
                    for (let i = l - 1; i >= 0; i--) {
                        // offsetHeight not of 0 means visible
                        if (focusElements[i].offsetHeight) {
                            this._focusElementEnd = focusElements[i];
                            i = focusElements.length;
                            break;
                        }
                    }
                    // this the zone this just a unique element, change mode for 'element'
                    if (this._focusElement === this._focusElementEnd) {
                        this.typeSelector = 'element';
                    }
                }
            }
            else {
                this._focusElement = focusElements[0];
                this.typeSelector = 'element';
            }
        }
        else {
            this._focusElement = null;
        }
    }
    /**
     * get instance, hightlight the focused element et show the template
     */
    _setFocus() {
        const instance = this._getInstance();
        if (instance) {
            const scrollY = window.pageXOffset;
            this._initStylingTemplate(instance);
            setTimeout(() => {
                if (this._focusElement && instance.zone) {
                    instance.marginZonePx = this._marginZonePx;
                    instance.hightlightZone(this._offsetCoordinates, scrollY - window.pageXOffset, this.animation, this.animationDelays, this._setFocusContinue.bind(this));
                }
                else {
                    this._setFocusContinue();
                }
            }, 20);
        }
    }
    _setFocusContinue() {
        const instance = this._getInstance();
        if (!this._display) {
            this._attachContentTemplate();
            this._initContentTemplate(instance);
        }
        setTimeout(() => {
            instance.hightlightZoneStyling(this._focusElement);
            this._updateElementPositions(instance);
        }, 0);
    }
    _updateElementPositions(instance) {
        if (WalkthroughComponent._walkthroughContainer && this._getInstance()) {
            setTimeout(() => {
                instance.contentBlockPosition(this._offsetCoordinates, this._alignContent, this._verticalAlignContent, this._contentSpacing, this._verticalContentSpacing);
                if (this._focusElement !== null && this._hasArrow) {
                    instance.arrowPosition(this._offsetCoordinates);
                }
                // add CSS to focusElement
                if (this.focusElementCSSClass && this._focusElement) {
                    this._renderer.addClass(this._focusElement, this.focusElementCSSClass);
                }
                setTimeout(() => {
                    this._getInstance().setHeight();
                    if (!this._readyHasBeenEmited) {
                        this._readyHasBeenEmited = true;
                        this.ready.emit(new WalkthroughEvent(this, this._focusElement));
                    }
                    const contentBlockNative = instance.contentBlock.nativeElement;
                    let scrollPos;
                    if (this._focusElement != null) {
                        const coordinatesContent = this._walkthroughService.retrieveCoordinates(contentBlockNative);
                        const coordinatesFocus = this._walkthroughService.retrieveCoordinates(this._focusElement);
                        // is content + focus higher than window ?
                        if (coordinatesContent.height + coordinatesFocus.height > window.innerHeight) {
                            // we scroll on content
                            contentBlockNative.scrollIntoView(true);
                            // we offset the window half of the content height
                            if (coordinatesContent.top > coordinatesFocus.top) {
                                // content below focusZone
                                scrollPos = -(coordinatesContent.height / 2);
                            }
                            else {
                                // content above focusZone
                                scrollPos = +(coordinatesContent.height / 2);
                            }
                        }
                        else {
                            // scroll on element higher minus minimal margin
                            if (coordinatesContent.top > coordinatesFocus.top) {
                                window.scrollTo(coordinatesFocus.left, coordinatesFocus.top);
                                scrollPos = -WalkthroughComponent.minimalMargin;
                            }
                            else {
                                contentBlockNative.scrollIntoView(true);
                                scrollPos = -WalkthroughComponent.minimalMargin;
                            }
                        }
                    }
                    else {
                        // no focus zone, scroll on content minus margin
                        contentBlockNative.scrollIntoView(true);
                        scrollPos = -WalkthroughComponent.minimalMargin;
                    }
                    window.scrollBy(0, scrollPos);
                }, 50);
            }, 0);
        }
    }
    /**
     * Attache the template in the contener, if a template is linked.
     */
    _attachContentTemplate() {
        if (this.contentTemplate) {
            this._attachWalkthroughContent(this.contentTemplate, this._getInstance());
        }
    }
    /**
     * init a partof styles of the contenaire
     */
    _initStylingTemplate(instance) {
        const hasHighlightZone = this._focusElement !== null;
        instance.parent = this;
        instance.open();
        instance.hasHighlightZone = hasHighlightZone;
        instance.hasClickable = hasHighlightZone && typeof this.focusClick === 'function';
        instance.hasHighlight = hasHighlightZone && this._hasHighlightAnimation;
        instance.hasBackdrop = this._hasBackdrop;
        instance.hasGlow = hasHighlightZone && this._hasGlow;
    }
    /**
     * init all datas of the contenaire
     */
    _initContentTemplate(instance) {
        const hasHighlightZone = this._focusElement !== null;
        instance.hasPrevious = this._hasPreviousStep(instance);
        instance.hasNext = this._hasNextStep(instance);
        instance.hasCloseButton = this._hasCloseButton;
        instance.hasCloseAnywhere = this._hasCloseAnywhere;
        instance.hasFinish = this._hasFinish || !instance.hasNext;
        instance.hasArrow = hasHighlightZone && this._hasArrow;
        instance.arrowColor = this.arrowColor;
        instance.radius = this.radius;
        instance.marginZone = this._marginZone ? this._marginZone.replace(/(\d+)/g, '$1px') : null;
        instance.marginZonePx = this._marginZonePx;
        instance.contentText = this.contentText;
        instance.contentStyle = this.contentStyle;
        instance.text = this.texts ? Object.assign(Object.assign({}, new WalkthroughText()), this.texts) : new WalkthroughText();
        this._show();
    }
    /**
     * check if there is a previous step enabled
     */
    _hasPreviousStep(instance) {
        if (instance.parent) {
            let current = instance.parent.previousStep;
            while (current) {
                if (!current.disabled) {
                    return true;
                }
                current = current.previousStep;
            }
        }
        return false;
    }
    /**
     * check if there is a next step enabled
     */
    _hasNextStep(instance) {
        if (instance.parent) {
            let current = instance.parent.nextStep;
            while (current) {
                if (!current.disabled) {
                    return true;
                }
                current = current.nextStep;
            }
        }
        return false;
    }
}
WalkthroughComponent._walkthroughContainer = null;
WalkthroughComponent._walkthroughContainerCreating = false;
WalkthroughComponent.minimalMargin = 60;
WalkthroughComponent.onOpen = new Subject();
WalkthroughComponent.onRefresh = new Subject();
WalkthroughComponent.onClose = new Subject();
WalkthroughComponent.onNavigate = new Subject();
WalkthroughComponent.onNavigatePrevious = new Subject();
WalkthroughComponent.onNavigateNext = new Subject();
WalkthroughComponent.onFinish = new Subject();
WalkthroughComponent.decorators = [
    { type: Component, args: [{
                selector: 'ng-walkthrough',
                template: ''
            },] }
];
/** @nocollapse */
WalkthroughComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver },
    { type: ApplicationRef },
    { type: Injector },
    { type: Renderer2 },
    { type: WalkthroughService }
];
WalkthroughComponent.propDecorators = {
    closed: [{ type: Output }],
    finished: [{ type: Output }],
    ready: [{ type: Output }],
    focusElementCSSClass: [{ type: Input }],
    rootElement: [{ type: Input }],
    focusElementSelector: [{ type: Input }],
    typeSelector: [{ type: Input }],
    focusClick: [{ type: Input }],
    radius: [{ type: Input }],
    previousStep: [{ type: Input }],
    nextStep: [{ type: Input }],
    texts: [{ type: Input }],
    contentTemplate: [{ type: Input }],
    contentText: [{ type: Input }],
    contentStyle: [{ type: Input }],
    marginZone: [{ type: Input }],
    arrowColor: [{ type: Input }],
    animation: [{ type: Input }],
    animationDelays: [{ type: Input }],
    notScrollOnResize: [{ type: Input }],
    id: [{ type: Input }],
    alignContent: [{ type: Input }],
    verticalAlignContent: [{ type: Input }],
    contentSpacing: [{ type: Input }],
    verticalContentSpacing: [{ type: Input }],
    closeButton: [{ type: Input }],
    closeAnywhere: [{ type: Input }],
    showArrow: [{ type: Input }],
    finishButton: [{ type: Input }],
    focusHighlightAnimation: [{ type: Input }],
    focusBackdrop: [{ type: Input }],
    focusGlow: [{ type: Input }],
    disabled: [{ type: Input }],
    scrollOnTarget: [{ type: Input }],
    visibilityCallback: [{ type: Input }],
    resize: [{ type: HostListener, args: ['window:resize',] }]
};

function throwWalkthroughContentAlreadyAttachedError() {
    throw Error('Attempting to attach walkthrough content after content is already attached');
}
const is_safari = /^((?!chrome|android).)*safari/i.test(navigator.userAgent);
class WalkthroughContainerComponent extends BasePortalOutlet {
    constructor(viewContainerRef, _walkthroughService, _renderer, _el) {
        super();
        this.viewContainerRef = viewContainerRef;
        this._walkthroughService = _walkthroughService;
        this._renderer = _renderer;
        this._el = _el;
        this.markerUrl = 'url(#wkt-arrow)';
        /* if a walkthrough is ongoing (paused or not) */
        this.ongoing = false;
        this.show = false;
        this.pause = false;
        // highlight zone
        this.hasHighlightZone = false;
        this.hasHighlight = false;
        this.hasBackdrop = false;
        this.hasGlow = false;
        // navigate
        this.hasPrevious = false;
        this.hasNext = false;
        this.hasFinish = false;
        this.hasCloseButton = false;
        this.hasCloseAnywhere = true;
        // arrow
        this.hasArrow = false;
        this.arrowMarkerDist = 7;
        this.marginZonePx = new WalkthroughMargin();
        // texts change / i18n
        this.text = new WalkthroughText();
    }
    // HostBinding
    get id() {
        return this.parent ? this.parent.id + '-container' : null;
    }
    get hide() {
        return !this.show;
    }
    get cursor() {
        return this.hasCloseAnywhere;
    }
    get backdrop() {
        return !this.hasHighlightZone && this.hasBackdrop;
    }
    click() {
        if (this.hasCloseAnywhere && this.show) {
            this.close();
        }
    }
    clickZone(event) {
        if (this.parent && this.hasClickable) {
            this.parent.focusClick(event, this);
        }
    }
    /**
     * Attach a ComponentPortal as content to this walkthrough container.
     * @param portal Portal to be attached as the walkthrough content.
     */
    attachComponentPortal(portal) {
        if (this._portalHost.hasAttached()) {
            throwWalkthroughContentAlreadyAttachedError();
        }
        // this._savePreviouslyFocusedElement();
        return this._portalHost.attachComponentPortal(portal);
    }
    /**
     * Attach a TemplatePortal as content to this walkthrough container.
     * @param portal Portal to be attached as the walkthrough content.
     */
    attachTemplatePortal(portal) {
        if (this._portalHost.hasAttached()) {
            throwWalkthroughContentAlreadyAttachedError();
        }
        // this._savePreviouslyFocusedElement();
        return this._portalHost.attachTemplatePortal(portal);
    }
    setHeight() {
        this._renderer.setStyle(this._el.nativeElement, 'height', this._walkthroughService.getDocumentHeight() + 'px');
    }
    hightlightZone(coordinate, scrollDiff, animation, animationDelays, continueFunction) {
        const element = this.zone.nativeElement;
        const zoneStyle = element.style;
        const style = window.getComputedStyle(element, null);
        if (animation === 'linear' && animationDelays > 0 && style.left !== 'auto') {
            this.hideOther = true;
            const fragment = 20;
            const intervale = animationDelays / fragment;
            const left = parseInt(style.left, 10);
            const top = scrollDiff + parseInt(style.top, 10);
            const width = parseInt(style.width, 10);
            const height = parseInt(style.height, 10);
            const partLeft = (coordinate.left - left) / fragment;
            const partTop = (coordinate.top - top) / fragment;
            const partWidth = (coordinate.width - width) / fragment;
            const partHeight = (coordinate.height - height) / fragment;
            let count = 0;
            this.show = true;
            zoneStyle.borderRadius = '50%';
            const timer = setInterval(() => {
                zoneStyle.left = left + partLeft * count + 'px';
                zoneStyle.top = top + partTop * count + 'px';
                zoneStyle.width = width + partWidth * count + 'px';
                zoneStyle.height = height + partHeight * count + 'px';
                if (count++ >= fragment) {
                    clearInterval(timer);
                    this.hideOther = false;
                    continueFunction();
                }
            }, intervale);
        }
        else {
            zoneStyle.left = coordinate.left - this.marginZonePx.left + 'px';
            zoneStyle.top = coordinate.top - this.marginZonePx.top + 'px';
            zoneStyle.width = coordinate.width + 'px';
            zoneStyle.height = coordinate.height + 'px';
            continueFunction();
        }
    }
    hightlightZoneStyling(element) {
        if (element) {
            const zoneStyle = this.zone.nativeElement.style;
            if (this.radius) {
                if (Number(this.radius) === parseFloat(this.radius)) {
                    // if is numeric, change in %
                    zoneStyle.borderRadius = this.radius + '%';
                }
                else if (this.radius === 'auto') {
                    // if mode auto
                    const elementStyle = window.getComputedStyle(element, null);
                    // borderRadius work only on Chrome, use TopLeft, TopRight... for Firefox/Egde/IE
                    zoneStyle.borderTopLeftRadius = elementStyle.borderTopLeftRadius;
                    zoneStyle.borderTopRightRadius = elementStyle.borderTopRightRadius;
                    zoneStyle.borderBottomLeftRadius = elementStyle.borderBottomLeftRadius;
                    zoneStyle.borderBottomRightRadius = elementStyle.borderBottomRightRadius;
                }
                else {
                    // if is numeric, change in %
                    zoneStyle.borderRadius = this.radius;
                }
            }
            else {
                zoneStyle.borderRadius = '';
            }
        }
    }
    contentBlockPosition(paramcoordinate, alignContent, verticalAlignContent, contentSpacing, verticalContentSpacing) {
        const element = this.contentBlock.nativeElement;
        const elementSize = this._walkthroughService.retrieveCoordinates(element);
        const width = elementSize.width + elementSize.margin.left + elementSize.margin.right;
        const height = elementSize.height + elementSize.margin.top + elementSize.margin.bottom;
        const coordinate = JSON.parse(JSON.stringify(paramcoordinate));
        coordinate.top -= this.marginZonePx.top;
        coordinate.left -= this.marginZonePx.left;
        coordinate.width += this.marginZonePx.left + this.marginZonePx.right;
        coordinate.height += this.marginZonePx.top + this.marginZonePx.bottom;
        // check if we've got the space to respect the alignContent attribute
        let notEnoughSpace = false;
        if (this.hasHighlightZone) {
            const spaceLeft = coordinate.left;
            const spaceRight = window.innerWidth - coordinate.left - coordinate.width;
            if (spaceLeft < width && spaceRight < width) {
                notEnoughSpace = true;
            }
            // if not enough space to respect the alignContent, content goes above/below
            if ((verticalAlignContent === 'top' ||
                verticalAlignContent === 'center' ||
                verticalAlignContent === 'bottom') &&
                !notEnoughSpace) {
                // change align center to left or right if not enough space to align center
                if (alignContent === 'center') {
                    const maxSpace = Math.max(spaceRight, spaceLeft);
                    if (maxSpace < width + (window.innerWidth - width) / 2) {
                        alignContent = spaceRight > spaceLeft ? 'right' : 'left';
                    }
                }
                else if ((alignContent === 'left' && spaceLeft < width) ||
                    (alignContent === 'right' && spaceRight < width)) {
                    verticalAlignContent =
                        verticalAlignContent === 'bottom' || coordinate.top < height ? 'below' : 'above';
                }
            }
        }
        // if not enough space on screen width, we center the content
        if (notEnoughSpace) {
            alignContent = 'center';
        }
        // position of content left/center/right
        element.style.right = '';
        element.style.left = '';
        if (alignContent === 'left') {
            element.style.left = '0';
            if (this.hasHighlightZone) {
                const space = coordinate.left - width;
                // handle contentSpacing
                if (contentSpacing && space > contentSpacing) {
                    element.style.left = coordinate.left - width - contentSpacing + 'px';
                }
            }
        }
        else if (alignContent === 'center') {
            element.style.left = window.innerWidth / 2 - width / 2 + 'px';
        }
        else if (alignContent === 'right') {
            element.style.right = '0';
            if (this.hasHighlightZone) {
                const space = window.innerWidth - coordinate.left - coordinate.width - width;
                // handle contentSpacing
                if (contentSpacing && space > contentSpacing) {
                    element.style.right = '';
                    element.style.left = coordinate.left + coordinate.width + contentSpacing + 'px';
                }
            }
        }
        if (this.hasHighlightZone) {
            // for arrow position
            const startLeft = this._walkthroughService.retrieveCoordinates(element).left + width / 2;
            this._arrowPosition =
                startLeft > coordinate.left - WalkthroughComponent.minimalMargin &&
                    startLeft < coordinate.left + coordinate.width + WalkthroughComponent.minimalMargin
                    ? 'topBottom'
                    : 'leftRight';
            // if there is enough place on the left or on the right, we consider verticalAlignContent, otherwise, we ignore it
            if (verticalAlignContent && !notEnoughSpace) {
                let space = 0;
                this._contentPosition = verticalAlignContent;
                switch (verticalAlignContent) {
                    case 'above':
                        space = coordinate.top;
                        if (space > verticalContentSpacing) {
                            element.style.top = coordinate.top - height - verticalContentSpacing + 'px';
                        }
                        else {
                            element.style.top = '0';
                        }
                        this._arrowPosition = 'topBottom';
                        break;
                    case 'top':
                        element.style.top = coordinate.top + 'px';
                        break;
                    case 'center':
                        element.style.top = coordinate.top + coordinate.height / 2 - height / 2 + 'px';
                        break;
                    case 'bottom':
                        element.style.top = coordinate.top + coordinate.height - height + 'px';
                        break;
                    case 'below':
                        space = this._walkthroughService.getDocumentHeight() - coordinate.top + coordinate.height;
                        if (space > verticalContentSpacing) {
                            element.style.top = coordinate.top + coordinate.height + verticalContentSpacing + 'px';
                        }
                        else {
                            element.style.top = this._walkthroughService.getDocumentHeight() - height + 'px';
                        }
                        this._arrowPosition = 'topBottom';
                        break;
                }
            }
            else {
                // position of content top/bottom
                if (verticalAlignContent === 'below' || coordinate.top < height) {
                    element.style.top = coordinate.top + coordinate.height + WalkthroughComponent.minimalMargin + 'px';
                    this._contentPosition = 'below';
                }
                else {
                    element.style.top = coordinate.top - height - WalkthroughComponent.minimalMargin + 'px';
                    this._contentPosition = 'above';
                }
            }
        }
        else {
            element.style.top = this._walkthroughService.getHeightOfPage() / 2 - height / 2 + 'px';
        }
    }
    arrowPosition(coordinate) {
        const contentBlockElement = this.contentBlock.nativeElement;
        const contentBlockCoordinates = this._walkthroughService.retrieveCoordinates(contentBlockElement);
        const realwidth = coordinate.width + this.marginZonePx.left + this.marginZonePx.right;
        const realheight = coordinate.height + this.marginZonePx.top + this.marginZonePx.bottom;
        // start point of the arrow (tail)
        let startLeft = contentBlockCoordinates.left + contentBlockCoordinates.width / 2;
        let startTop = contentBlockCoordinates.top + contentBlockCoordinates.height;
        // start point of the curve of the arrow
        let centerTop;
        let centerLeft;
        // end point of the arrow (head)
        let endLeft = coordinate.left - this.marginZonePx.left;
        let endTop = coordinate.top - this.marginZonePx.top;
        switch (this._contentPosition) {
            case 'top':
            case 'center':
            case 'bottom':
                if (contentBlockCoordinates.left > coordinate.left) {
                    startLeft = contentBlockCoordinates.left;
                }
                else {
                    startLeft = contentBlockCoordinates.left + contentBlockCoordinates.width;
                }
                startTop -= contentBlockCoordinates.height / 2;
                break;
            case 'below':
                startTop -= contentBlockCoordinates.height;
                break;
        }
        if (this._arrowPosition === 'topBottom') {
            endLeft += realwidth / 2;
            if (this._contentPosition === 'below') {
                endTop += realheight + 6;
            }
            else {
                endTop -= 6;
            }
            centerLeft = (startLeft + endLeft) / 2;
            centerTop = (startTop + endTop) / 2;
            // we make the curve start nearer the tail of the arrow
            if (startTop < centerTop) {
                centerTop = centerTop - Math.abs(startTop - centerTop) / 2;
            }
            else {
                centerTop = centerTop + Math.abs(startTop - centerTop) / 2;
            }
            this.arrowPath =
                `M${startLeft},${startTop} Q${startLeft},${centerTop} ${centerLeft},${centerTop} ` +
                    `Q${endLeft},${centerTop} ${endLeft},${endTop}`;
        }
        else {
            if (startLeft > coordinate.left) {
                endLeft += realwidth + this.arrowMarkerDist;
            }
            else {
                endLeft -= this.arrowMarkerDist;
            }
            endTop += realheight / 2;
            centerLeft = (startLeft + endLeft) / 2;
            centerTop = (startTop + endTop) / 2;
            // we make the curve start nearer the tail of the arrow
            if (startLeft < centerLeft) {
                centerLeft = centerLeft - Math.abs(startLeft - centerLeft) / 2;
            }
            else {
                centerLeft = centerLeft + Math.abs(startLeft - centerLeft) / 2;
            }
            let directStartLeft = startLeft;
            let directStartTop = startTop;
            if (this._contentPosition === 'top' || this._contentPosition === 'bottom') {
                directStartLeft = contentBlockCoordinates.left + contentBlockCoordinates.width / 2;
                directStartTop =
                    this._contentPosition === 'top'
                        ? contentBlockCoordinates.top + contentBlockCoordinates.height
                        : contentBlockCoordinates.top;
                // we use direct curve only if the arrow don't cross the content, otherwise, we use double curved
                if ((this._contentPosition === 'top' && directStartTop < endTop - WalkthroughComponent.minimalMargin) ||
                    (this._contentPosition === 'bottom' && directStartTop > endTop + WalkthroughComponent.minimalMargin)) {
                    this.arrowPath = `M${directStartLeft},${directStartTop} Q${directStartLeft},${endTop} ${endLeft},${endTop}`;
                }
                else {
                    this.arrowPath =
                        `M${startLeft},${startTop} Q${centerLeft},${startTop} ${centerLeft},${centerTop} ` +
                            `Q${centerLeft},${endTop} ${endLeft},${endTop}`;
                }
            }
            else {
                this.arrowPath = `M${directStartLeft},${directStartTop} Q${directStartLeft},${endTop} ${endLeft},${endTop}`;
            }
        }
    }
    /**
     * stop the walkthrough : hide the container and change to pause at true
     */
    stop() {
        if (this.parent && !this.pause && this.show) {
            this.show = false;
            this.pause = true;
        }
    }
    /**
     * continue the walkthrough if is stopped : show the container and change to pause at false
     */
    continue() {
        if (this.parent && this.pause) {
            this.show = true;
            this.pause = false;
            this.parent.refresh();
        }
    }
    open() {
        // change markerUrl on Safari
        // related to
        // https://gist.github.com/leonderijke/c5cf7c5b2e424c0061d2
        // http://stackoverflow.com/a/18265336/796152
        // http://www.w3.org/TR/SVG/linking.html
        if (is_safari) {
            this.markerUrl = 'url(' + window.location.href + '#wkt-arrow)';
        }
        this.show = true;
    }
    previous() {
        this.close(false, false);
        // we check if previous walkthrough is not disabled
        let current = this.parent;
        while (current) {
            if (current.previousStep && !current.previousStep.disabled) {
                current.loadPrevioustStep();
                return;
            }
            else {
                if (!current.previousStep) {
                    break;
                }
                current = current.previousStep;
            }
        }
        // no more previous walkthrough enabled, we quit the walkthrough
        this.parent = current;
        this.close(true, true);
    }
    next() {
        this.close(false, false);
        // we check if next walkthrough is not disabled
        let current = this.parent;
        while (current) {
            if (current.nextStep && !current.nextStep.disabled) {
                current.loadNextStep();
                return;
            }
            else {
                if (!current.nextStep) {
                    break;
                }
                current = current.nextStep;
            }
        }
        // no more next walkthrough enabled, we quit the walkthrough
        this.parent = current;
        this.close(true, true);
    }
    close(finishLink = false, closeWalkthrough = true) {
        // remove content
        this._portalHost.dispose();
        // hide
        this.show = false;
        if (this.parent) {
            this.parent._hide(finishLink, closeWalkthrough);
        }
    }
}
WalkthroughContainerComponent.decorators = [
    { type: Component, args: [{
                selector: 'walkthrough-container',
                template: "<div class=\"wkt-container\">\n    <div\n        class=\"wkt-zone\"\n        #zone\n        (click)=\"clickZone($event)\"\n        [class.hide]=\"!hasHighlightZone\"\n        [class.backdrop]=\"hasBackdrop\"\n        [class.glow]=\"hasGlow\"\n        [class.clickable]=\"hasClickable\"\n        [class.highlight]=\"hasHighlight\"\n        [style.padding]=\"marginZone\"\n    ></div>\n    <div class=\"wkt-content-block\" [class.hide]=\"hideOther\" [class.darken]=\"contentStyle === 'draken'\" #contentBlock>\n        <button type=\"button\" class=\"wkt-close\" *ngIf=\"hasCloseButton\" (click)=\"close()\">\u2716</button>\n        <div class=\"wkt-content\">\n            <p *ngIf=\"contentText\">{{ contentText }}</p>\n            <ng-template cdkPortalHost></ng-template>\n        </div>\n        <div class=\"wkt-navigate\" *ngIf=\"hasPrevious || hasNext || hasFinish\">\n            <button type=\"button\" class=\"wkt-previous-link\" *ngIf=\"hasPrevious\" (click)=\"previous()\">\n                {{ text.previous }}\n            </button>\n            <button type=\"button\" class=\"wkt-next-link\" *ngIf=\"hasNext\" (click)=\"next()\">{{ text.next }}</button>\n            <button type=\"button\" class=\"wkt-finish-link\" *ngIf=\"hasFinish\" (click)=\"close(true)\">\n                {{ text.close }}\n            </button>\n        </div>\n    </div>\n    <div *ngIf=\"hasArrow\" [class.hide]=\"hideOther\">\n        <svg version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" width=\"100%\" height=\"100%\">\n            <defs>\n                <marker\n                    id=\"wkt-arrow\"\n                    viewBox=\"0 0 10 10\"\n                    refX=\"8\"\n                    refY=\"5\"\n                    markerUnits=\"strokeWidth\"\n                    orient=\"auto\"\n                    markerWidth=\"10\"\n                    markerHeight=\"10\"\n                >\n                    <polyline points=\"0,0 10,5 0,10 5,5\" stroke-width=\"0\" [attr.fill]=\"arrowColor || '#FFF'\" />\n                </marker>\n            </defs>\n            <path\n                class=\"wkt-arrow-path\"\n                [attr.d]=\"arrowPath\"\n                [attr.stroke]=\"arrowColor || '#FFF'\"\n                [attr.marker-end]=\"markerUrl\"\n            />\n        </svg>\n    </div>\n</div>\n",
                styles: [":host{display:block;position:absolute;z-index:1;top:0;left:0;width:99.9%;min-height:100%}:host.hide{display:none}:host.cursor{cursor:pointer}:host.backdrop{background-color:rgba(0,0,0,.6)}button{background:none;border:0;color:#fff}.wkt-zone{position:absolute}.wkt-zone.hide{display:none}.wkt-zone.backdrop{box-shadow:0 0 0 2200px rgba(0,0,0,.6)}.wkt-zone.glow{box-shadow:0 0 4px 2px #ff6,0 0 9px 0 #ff6,0 0 36px #ff6}.wkt-zone.clickable{cursor:pointer}.wkt-zone.highlight{-webkit-animation:highlight .45s 4;animation:highlight .45s 4}.wkt-container{position:relative}.wkt-content-block{position:absolute;margin:1em;padding:6px;z-index:1;color:#fff;min-width:calc(320px - 2em)}.wkt-content-block.hide{display:none}.wkt-content-block.darken{background-color:rgba(0,0,0,.6)}.wkt-close+.wkt-content{margin-top:1.3em}.wkt-close{position:absolute;right:1em;cursor:pointer}.wkt-navigate{display:flex;flex-wrap:wrap;justify-content:center}.wkt-navigate>button{padding:6px;font-size:120%}.wkt-navigate>button:hover{cursor:pointer;text-decoration:underline}.wkt-previous-link:before{content:\"<< \"}.wkt-next-link:after{content:\" >>\"}div.hide{display:none}svg{overflow:visible;position:absolute;top:-1px;left:-1px;border:1px solid transparent}#wkt-arrow>path{stroke-width:0}.wkt-arrow-path{stroke-width:2px;fill:none}@-webkit-keyframes highlight{0%{background-color:none}50%{background-color:hsla(0,0%,100%,.8)}to{background-color:none}}@keyframes highlight{0%{background-color:none}50%{background-color:hsla(0,0%,100%,.8)}to{background-color:none}}@media screen and (min-height:1920px),screen and (min-width:1920px){.wkt-zone.backdrop{box-shadow:0 0 0 4400px rgba(0,0,0,.6)}}@media screen and (min-height:3840px),screen and (min-width:3840px){.wkt-zone.backdrop{box-shadow:0 0 0 8800px rgba(0,0,0,.6)}}.wkt-zone.backdrop,_:default:not(:root:root){box-shadow:0 0 0 2000px rgba(0,0,0,.6)!important}"]
            },] }
];
/** @nocollapse */
WalkthroughContainerComponent.ctorParameters = () => [
    { type: ViewContainerRef },
    { type: WalkthroughService },
    { type: Renderer2 },
    { type: ElementRef }
];
WalkthroughContainerComponent.propDecorators = {
    _portalHost: [{ type: ViewChild, args: [CdkPortalOutlet,] }],
    content: [{ type: ViewChild, args: ['content',] }],
    contentBlock: [{ type: ViewChild, args: ['contentBlock',] }],
    zone: [{ type: ViewChild, args: ['zone',] }],
    id: [{ type: HostBinding, args: ['attr.id',] }],
    hide: [{ type: HostBinding, args: ['class.hide',] }],
    cursor: [{ type: HostBinding, args: ['class.cursor',] }],
    backdrop: [{ type: HostBinding, args: ['class.backdrop',] }],
    click: [{ type: HostListener, args: ['click',] }]
};

let nextUniqueId = 0;
class WalkthroughFlowComponent {
    constructor() {
        this.closed = new EventEmitter();
        this.finished = new EventEmitter();
        this.contentStyle = 'draken';
        this.marginZone = null;
        this._uid = `walkthrough-flow-${nextUniqueId++}`;
    }
    get id() {
        return this._id;
    }
    set id(value) {
        this._id = value || this._uid;
    }
    ngAfterViewInit() {
        setTimeout(() => {
            this.init();
        });
    }
    init() {
        let prevComp = null;
        this.walkthroughComponents.forEach((walkthrough) => {
            // navigation auto (ignore previousStep/nextStep on the WalkthroughComponent)
            if (prevComp) {
                walkthrough.previousStep = prevComp;
                prevComp.nextStep = walkthrough;
            }
            prevComp = walkthrough;
            // transmition (only if different from default)
            if (this.closed) {
                walkthrough.closed = this.closed;
            }
            if (this.finished) {
                walkthrough.finished = this.finished;
            }
            if (!walkthrough.contentStyle && this.contentStyle) {
                walkthrough.contentStyle = this.contentStyle;
            }
            if (!walkthrough.arrowColor && this.arrowColor) {
                walkthrough.arrowColor = this.arrowColor;
            }
            if (!walkthrough.marginZone && this.marginZone) {
                walkthrough.marginZone = this.marginZone;
            }
            if (!walkthrough.showArrow && booleanValue(this.showArrow)) {
                walkthrough.showArrow = this.showArrow;
            }
            if (!walkthrough.rootElement && this.rootElement) {
                walkthrough.rootElement = this.rootElement;
            }
            if (!walkthrough.closeButton && booleanValue(this.closeButton)) {
                walkthrough.closeButton = this.closeButton;
            }
            if (walkthrough.closeAnywhere && !booleanValue(this.closeAnywhere)) {
                walkthrough.closeAnywhere = this.closeAnywhere;
            }
            if (!walkthrough.finishButton && booleanValue(this.finishButton)) {
                walkthrough.finishButton = this.finishButton;
            }
            if (!walkthrough.focusBackdrop && booleanValue(this.focusBackdrop)) {
                walkthrough.focusBackdrop = this.focusBackdrop;
            }
            if (!walkthrough.focusGlow && booleanValue(this.focusGlow)) {
                walkthrough.focusGlow = this.focusGlow;
            }
            if (!walkthrough.notScrollOnResize && booleanValue(this.notScrollOnResize)) {
                walkthrough.notScrollOnResize = this.notScrollOnResize;
            }
            if (!walkthrough.radius && this.radius) {
                walkthrough.radius = this.radius;
            }
            if (!walkthrough.texts && this.texts) {
                walkthrough.texts = this.texts;
            }
        });
        // navigation auto (close on last step)
        prevComp.finishButton = true;
    }
    start() {
        this.walkthroughComponents.first.open();
    }
}
WalkthroughFlowComponent.decorators = [
    { type: Component, args: [{
                selector: 'ng-walkthrough-flow',
                template: ''
            },] }
];
/** @nocollapse */
WalkthroughFlowComponent.ctorParameters = () => [];
WalkthroughFlowComponent.propDecorators = {
    walkthroughComponents: [{ type: ContentChildren, args: [WalkthroughComponent,] }],
    id: [{ type: Input }],
    closed: [{ type: Output }],
    finished: [{ type: Output }],
    contentStyle: [{ type: Input }],
    arrowColor: [{ type: Input }],
    marginZone: [{ type: Input }],
    showArrow: [{ type: Input }],
    rootElement: [{ type: Input }],
    closeButton: [{ type: Input }],
    closeAnywhere: [{ type: Input }],
    finishButton: [{ type: Input }],
    focusBackdrop: [{ type: Input }],
    focusGlow: [{ type: Input }],
    radius: [{ type: Input }],
    notScrollOnResize: [{ type: Input }],
    texts: [{ type: Input }]
};

class WalkthroughModule {
}
WalkthroughModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, PortalModule],
                declarations: [WalkthroughFlowComponent, WalkthroughComponent, WalkthroughContainerComponent],
                exports: [WalkthroughFlowComponent, WalkthroughComponent, WalkthroughContainerComponent],
                entryComponents: [WalkthroughContainerComponent],
                providers: [WalkthroughService],
            },] }
];

/*
 * Public API Surface of angular-walkthrough
 */

/**
 * Generated bundle index. Do not edit.
 */

export { WalkthroughComponent, WalkthroughContainerComponent, WalkthroughEvent, WalkthroughFlowComponent, WalkthroughMargin, WalkthroughModule, WalkthroughService, WalkthroughText, booleanValue, throwWalkthroughContentAlreadyAttachedError };
//# sourceMappingURL=angular-walkthrough.js.map
